==> nodeinfo-0.3.1.tar.gz: nodeinfo-go/extension/socialhome.go <==
package extension

import (
	"encoding/json"
	"fmt"
	"io"
	"net/http"

	"codeberg.org/thefederationinfo/nodeinfo-go"
	known_extension "codeberg.org/thefederationinfo/nodeinfo_metadata_survey"
)

func init() {
	nodeinfo.RegisterExtension(&socialhome{})
}

type socialhome struct{}

func (socialhome) Name() string {
	return "socialhome"
}

type socialhomeInfo struct {
	Usage struct {
		Users struct {
			ActiveWeek int64 `json:"activeWeek"`
		} `json:"users"`
	} `json:"usage"`
	Relay        *string `json:"relay"`
	Organization struct {
		Name    *string `json:"name"`
		Contact *string `json:"contact"`
		Account *string `json:"account"`
	} `json:"organization"`
}

func (socialhome) Extend(c *http.Client, domain string, n *nodeinfo.Node) error {
	if n.Metadata == nil {
		n.Metadata = make(map[string]interface{})
	}

	_, adminsOK := n.Metadata[known_extension.KeyAdmins]

	resp, err := c.Get(fmt.Sprintf("%s://%s/.well-known/x-nodeinfo2", "https", domain))
	if err != nil {
		return err
	}

	// no x-nodeinfo server found
	if resp.StatusCode == http.StatusNotFound {
		return nil
	} else if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("unexpected status code on lookup")
	}

	rawResp, err := io.ReadAll(resp.Body)
	if err != nil {
		return err
	}
	node := &socialhomeInfo{}
	if err := json.Unmarshal(rawResp, node); err != nil {
		return err
	}

	toStr := func(s *string) string {
		if s == nil {
			return ""
		}
		return *s
	}

	if !adminsOK && (node.Organization.Account != nil ||
		node.Organization.Contact != nil ||
		node.Organization.Name != nil) {
		n.Metadata[known_extension.KeyAdmins] = []known_extension.NodeAdmin{{
			Name:    toStr(node.Organization.Name),
			Account: toStr(node.Organization.Account),
			Email:   toStr(node.Organization.Contact),
		}}
	}

	if _, ok := n.Metadata["relay"]; !ok && node.Relay != nil {
		n.Metadata["relay"] = *node.Relay
	}

	if _, ok := n.Metadata["usageUsersActiveWeek"]; !ok {
		n.Metadata["usageUsersActiveWeek"] = node.Usage.Users.ActiveWeek
	}

	return nil
}

==> nodeinfo-1.0.0.tar.gz: nodeinfo-go/extension/socialhome.go <==
package extension

import (
	"encoding/json"
	"fmt"
	"io"
	"net/http"
	"strconv"

	"codeberg.org/thefederationinfo/nodeinfo-go"
	known_extension "codeberg.org/thefederationinfo/nodeinfo_metadata_survey"
)

func init() {
	nodeinfo.RegisterExtension(&socialhome{})
}

type socialhome struct{}

func (socialhome) Name() string {
	return "socialhome"
}

type socialhomeInfo struct {
	Usage struct {
		Users struct {
			ActiveWeek int64 `json:"activeWeek"`
		} `json:"users"`
	} `json:"usage"`
	Relay        *string `json:"relay"`
	Organization struct {
		Name    *string `json:"name"`
		Contact *string `json:"contact"`
		Account *string `json:"account"`
	} `json:"organization"`
}

func (socialhome) Extend(c *http.Client, domain string, n *nodeinfo.Node) error {
	n.Version = "2.2"
	if n.Metadata == nil {
		n.Metadata = make(map[string]interface{})
	}

	_, adminsOK := n.Metadata[known_extension.KeyAdmins]

	resp, err := c.Get(fmt.Sprintf("%s://%s/.well-known/x-nodeinfo2", "https", domain))
	if err != nil {
		return err
	}

	// no x-nodeinfo server found
	if resp.StatusCode == http.StatusNotFound {
		return nil
	} else if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("unexpected status code on lookup")
	}

	rawResp, err := io.ReadAll(resp.Body)
	if err != nil {
		return err
	}
	node := &socialhomeInfo{}
	if err := json.Unmarshal(rawResp, node); err != nil {
		return err
	}

	toStr := func(s *string) string {
		if s == nil {
			return ""
		}
		return *s
	}

	if !adminsOK && (node.Organization.Account != nil ||
		node.Organization.Contact != nil ||
		node.Organization.Name != nil) {
		n.Metadata[known_extension.KeyAdmins] = []known_extension.NodeAdmin{{
			Name:    toStr(node.Organization.Name),
			Account: toStr(node.Organization.Account),
			Email:   toStr(node.Organization.Contact),
		}}
	}

	if _, ok := n.Metadata["relay"]; !ok && node.Relay != nil {
		n.Metadata["relay"] = *node.Relay
	}

	// if nodeinfo2.2 does not provide it, we first try to get it from socialhomeInfo
	// if this don't work we try to lookup and convert from Metadata
	if n.Usage.Users.ActiveWeek == 0 {
		if node.Usage.Users.ActiveWeek != 0 {
			n.Usage.Users.ActiveWeek = node.Usage.Users.ActiveWeek
		} else if metadataUsersWeek, ok := n.Metadata["usageUsersActiveWeek"]; ok {
			if usersWeek, err := strconv.ParseInt(fmt.Sprint(metadataUsersWeek), 10, 64); err == nil {
				n.Usage.Users.ActiveWeek = usersWeek
				delete(n.Metadata, "usageUsersActiveWeek")
			}
		}
	}

	return nil
}
